--[[============================================================================
main.lua
============================================================================]]--
require 'midi_device'
require 'midi_control'
require 'gui'

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  
end

-- Read from the manifest.xml file.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value

pitch_bend_max_range = 127 -- default is 127 with most, but some also seem to support up to 65535
selected_pitch_device = nil
selected_instrument = 1
sample_transpose_mode = 1       --Mode 1 = pitch envelope transpose, mode 2 is sample transpose.
instruments = {}
change_from_renoise = false     --Toggle to prevent feedbacks to the GUI controls when a value is changed by Renoise
change_from_controller = false  --Toggle to call routines required when the midi pitch bend controller is touched.
change_from_tool = false        --Toggle to prevent feedbacks to the GUI controls when a value is changed by the tool
change_from_config = false      --Toggle to prevent changes done through the config to affect current instrument settings
pitch_range = 3
pitch_range_max = 12
triggered_track = nil
triggered_device = nil
device_notifiers = {}  --Functions to the device notifiers
soffset = {} --Sample transpose offset values (what was the initial value and from there transpose up/down)
instrument_links = {} --Is the instrument linked to a device? Optimizing notifier speedups.

--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

function monitor_pitch()
  --Set global notifiers, some notifiers are set outside this function or in
  --specific functions if the notifier needs to be dynamically assigned.
  local song = renoise.song()

  --cover dsp device selection
  if not song.selected_device_observable:has_notifier(watch_device_monitor) then
    song.selected_device_observable:add_notifier(watch_device_monitor)
  end

  --Cover instrument selections
  if not song.selected_instrument_index_observable:has_notifier(set_pitch_device_notifier) then
    song.selected_instrument_index_observable:add_notifier(set_pitch_device_notifier) 
  end  

  --Cover instrument changes (deletions, insertions)
  if not song.instruments_observable:has_notifier(recover_instrument_instrument_links) then
    song.instruments_observable:add_notifier(recover_instrument_instrument_links) 
  end  

  --Cover track changes
  if not song.tracks_observable:has_notifier(set_pitch_device_notifiers) then
    song.tracks_observable:add_notifier(set_pitch_device_notifiers)
  end
  
  --cover document changes
  if not renoise.tool().app_release_document_observable:has_notifier(clean_up) then
    renoise.tool().app_release_document_observable:add_notifier(clean_up)
  end


  
end

---------------------------------------------------------------------------------------------

function clean_up()
  --Called before the song is unloaded
  local song = renoise.song()
  --Let's clean up everything!
  if song.selected_device_observable:has_notifier(watch_device_monitor) then
    song.selected_device_observable:remove_notifier(watch_device_monitor)
  end
  
  if song.selected_instrument_index_observable:has_notifier(set_pitch_device_notifier) then
    song.selected_instrument_index_observable:remove_notifier(set_pitch_device_notifier) 
  end  

  if song.instruments_observable:has_notifier(set_pitch_device_notifier) then
    song.instruments_observable:remove_notifier(set_pitch_device_notifier) 
  end  
  
  if song.tracks_observable:has_notifier(set_pitch_device_notifiers) then
    song.tracks_observable:remove_notifier(set_pitch_device_notifiers)
  end

  for x = 1, #song.instruments do
    if instrument_links[x] == true then
      for y = 1,#song.instruments[x].samples do
        if song.instruments[x].samples[y].transpose_observable:has_notifier(reset_sample_offsets) then
          song.instruments[x].samples[y].transpose_observable:remove_notifier(reset_sample_offsets)
        end
      end
    end
  end
  
  for x = 1, #song.tracks do
    if song.tracks[x].devices_observable:has_notifier(set_pitch_device_notifiers) then
      song.tracks[x].devices_observable:remove_notifier(set_pitch_device_notifiers)
    end  
  
    for y = 1, #song.tracks[x].devices do
      local func_ref = 't'..tostring(x)..'d'..tostring(y)
      
      if device_notifiers[func_ref] ~= nil then
      
        if song.tracks[x].devices[y].parameters[1].value_observable:has_notifier(device_notifiers[func_ref]) then
          song.tracks[x].devices[y].parameters[1].value_observable:remove_notifier(device_notifiers[func_ref])
        end
      end
    end
  end
  
  device_notifiers = {}

  
end

---------------------------------------------------------------------------------------------

function watch_device_monitor()
  --Called when another DSP device is selected
  local song = renoise.song()
  local track=song.selected_track_index
  local track_device_idx = song.selected_device_index
  local track_device = song.tracks[track].devices[track_device_idx]
  
  if track_device ~= nil then
    if string.find(string.lower(track_device.display_name), "pitch device") ~= nil then
      selected_pitch_device = track_device
      if get_pitch_device_instrument(track_device) ~= nil then
        renoise.song().selected_instrument_index = get_pitch_device_instrument(track_device)
        selected_midi_device = get_pitch_device_midi_link(track_device)
      end
      return true
    end  
  end
  
  selected_pitch_device = nil
  
  return false
  
end

---------------------------------------------------------------------------------------------

function get_pitch_device_midi_link(pitch_device)
--Read the device title and get the midi device linked to the device.
  if pitch_device == nil then
    return 1
  end
  
  local properties = pitch_device.display_name:split("[^,]+")
  
  if properties ~= nil then
    if string.lower(properties[1]) ~= "pitch device" or properties[5] == nil then
      return 1
    end

    if #properties > 0 then
      local linked_midi_device = tonumber(properties[5])
      if linked_midi_device < 1 then linked_midi_device = 1 end
      if linked_midi_device > #renoise.Midi.available_input_devices() then linked_midi_device = 1 end
      return linked_midi_device
    end
  end
  return 1
end

---------------------------------------------------------------------------------------------

function get_pitch_device_instrument(pitch_device)
--Read the device title and get the instrument linked to this device.
  if pitch_device == nil then
    return nil
  end
  
  local properties = pitch_device.display_name:split("[^,]+")
  
  if properties ~= nil then
    if string.lower(properties[1]) ~= "pitch device" or properties[2] == nil then
      return nil
    end

    if #properties > 0 then
      local ins_num = tonumber(properties[2])

      if tonumber(properties[2]) < 1 then ins_num = 1 end
      if ins_num > #renoise.song().instruments then ins_num = 1 end
      return ins_num
    end
  end
  return nil
end

---------------------------------------------------------------------------------------------

function set_pitch_device_notifiers()
  --all available pitch-devices are getting a notifier
  --We are generating dynamic functions that are given a notifier.
  --This is pretty much how you can deal with dynamic functionality
  --No need for _G['funcname_ref'] but it is practically the same.
  --Moving and erasing or adding tracks is covered by this function.
  local song = renoise.song()
  
  --Let's clean up first to prevent the notifier array from getting too clunky
  for x = 1, #song.tracks do
    if song.tracks[x].devices_observable:has_notifier(set_pitch_device_notifiers) then
      song.tracks[x].devices_observable:remove_notifier(set_pitch_device_notifiers)
    end  
  
    for y = 1, #song.tracks[x].devices do
      local func_ref = 't'..tostring(x)..'d'..tostring(y)
      
      if device_notifiers[func_ref] ~= nil then
        --We'll start by removing the old notifier
        if song.tracks[x].devices[y].parameters[1].value_observable:has_notifier(device_notifiers[func_ref]) then
          song.tracks[x].devices[y].parameters[1].value_observable:remove_notifier(device_notifiers[func_ref])
        end
      end
    end
  end
  
  --then we simply swat the array
  device_notifiers = {}
  
  --populating the new notifier functions
  for x = 1, #song.tracks do
    --Cover DSP changes 
    if not song.tracks[x].devices_observable:has_notifier(set_pitch_device_notifiers) then
      song.tracks[x].devices_observable:add_notifier(set_pitch_device_notifiers)
    end  

    for y = 1, #song.tracks[x].devices do
      local func_ref = 't'..tostring(x)..'d'..tostring(y)
      
      if string.find(string.lower(song.tracks[x].devices[y].display_name), "pitch device") ~= nil then
        --Putting functions inside an array this way rather than creating a global name-space variable
        --has a few advantages:
        --all functions are referenced from the same spot and the table can be simply erased ;)
         device_notifiers[func_ref] = function()
                                        change_from_tool = true 
                                          triggered_track = x
                                          triggered_device = y
                                          get_pitch_device_properties(song.tracks[x].devices[y])
                                          set_pitchbend_slider (x,y, selected_instrument, sample_transpose_mode, pitch_range )
                                        change_from_tool = false 
                                      end

         if not song.tracks[x].devices[y].parameters[1].value_observable:has_notifier(device_notifiers[func_ref]) then
            song.tracks[x].devices[y].parameters[1].value_observable:add_notifier(device_notifiers[func_ref])
         end
      end
    end
  end
  set_pitch_device_notifier()
  
end

---------------------------------------------------------------------------------------------

function set_pitch_device_notifier()
 --Called when another instrument gets selected, let's focus the midi controller to the device 
 --that is linked to the instrument
  local song = renoise.song()
  local found = 0
  change_from_tool = true

  for x = 1, #song.tracks do
  
    for y = 1, #song.tracks[x].devices do
    
      if string.find(string.lower(song.tracks[x].devices[y].display_name), "pitch device") ~= nil then
      
        get_pitch_device_properties(song.tracks[x].devices[y])
        
        if selected_instrument == song.selected_instrument_index then
          found = 1
          selected_pitch_device = song.tracks[x].devices[y]
          monitor_pitch_bend_controller()
          break
        end
      end
    end
    
    if found == 1 then
      break
    end
  end
  change_from_tool = false
  
end

---------------------------------------------------------------------------------------------

function recover_instrument_instrument_links()
  --called when something in the instrument list is changing.
  --(deletion, insertion, new song)
  get_instruments()
  set_pitch_device_notifier()
end

---------------------------------------------------------------------------------------------

function link_sample_transpose(instrument)
  --Called when the sample transpose is turned on for an instrument.
  local samples = renoise.song().instruments[instrument].samples

  if instrument_links[instrument] then

    for x = 1, #samples do

      if not samples[x].transpose_observable:has_notifier(reset_sample_offsets) then
        samples[x].transpose_observable:add_notifier(reset_sample_offsets)
      end
    end
  else
    for x = 1, #samples do

      if samples[x].transpose_observable:has_notifier(reset_sample_offsets) then
        samples[x].transpose_observable:remove_notifier(reset_sample_offsets)
      end
    end
  end
end
---------------------------------------------------------------------------------------------

function reset_sample_offsets(instrument_index)
  --Called when something changes in the instrument list or another
  --instrument gets selected
  if change_from_tool then
    return
  end
  local song = renoise.song()
  local samples = song.instruments[song.selected_instrument_index].samples
  if instrument_index ~= nil then
    samples = song.instruments[instrument_index].samples
  end
  if instrument_links[song.selected_instrument_index] then
    soffset = {} --resetting the sample offset values for the sample transpose ;)

    for x = 1, #samples do
      soffset[x] = samples[x].transpose
    end
  end
  
end

---------------------------------------------------------------------------------------------

function monitor_pitch_bend_controller()
  --Start the midi device engine using the programmed midi device from the pitch device
  get_midi_devices()
  midi_engine('stop')
  midi_engine('start')
end

---------------------------------------------------------------------------------------------

function get_midi_devices()
--This function sets a specific midi device that will be listened to in the midi_control.lua
--you can scan for available devices and let the user pick one and then listen to that device.
--Change your midi device by selecting the array-number of your device (unquote the print function to see yours in the terminal)
  device_list[1] = "None"
  for x = 1, #renoise.Midi.available_input_devices() do
    device_list[x+1] = renoise.Midi.available_input_devices()[x]
  end
  --rprint(device_list)
  if selected_midi_device == nil then
    selected_midi_device = 1
  end
  
end

---------------------------------------------------------------------------------------------

function get_instruments()
  local instrument_list = renoise.song().instruments

  for _ = 1, #instrument_list do
    instruments[_] = instrument_list[_].name
    instrument_links[_] = false
  end
  if pa_gui ~= nil then
    pa_gui.views['selected_instrument'].items = instruments
  end  
end

---------------------------------------------------------------------------------------------

function set_pitch_device_properties(created)
  --Called when a new device is created or when properties are altered in the dialog.
  local d_name = 'Pitch Device'
  local song = renoise.song()
  
  if pa_gui ~= nil then
    if not created then
      d_name = d_name..','..pa_gui.views['selected_instrument'].value
    else
      d_name = d_name..','..tostring(renoise.song().selected_instrument_index)
      local linked_instrument = renoise.song().selected_instrument_index
      local samples = song.instruments[linked_instrument].samples
      for x = 1, #samples do
        if soffset[x] == nil then
          soffset[x] = samples[x].transpose
        end
      end      
    end
    d_name = d_name..','..pa_gui.views['sample_transpose_mode'].value
    d_name = d_name..','..pa_gui.views['pitch_range'].value
    d_name = d_name..','..pa_gui.views['selected_midi_device'].value
    d_name = d_name..','..pa_gui.views['selected_channel'].value
    if pa_gui.views['sample_transpose_mode'].value ~= 1 then --Sample transpose
      instrument_links[pa_gui.views['selected_instrument'].value] = true
      link_sample_transpose(pa_gui.views['selected_instrument'].value)
      reset_sample_offsets(pa_gui.views['selected_instrument'].value)
    else
      instrument_links[pa_gui.views['selected_instrument'].value] = false
      link_sample_transpose(pa_gui.views['selected_instrument'].value)
    end
  else
    d_name = d_name..','..tostring(renoise.song().selected_instrument_index)
    d_name = d_name..',1'
    d_name = d_name..',3'
    d_name = d_name..',1'
    d_name = d_name..',0'
  end
    
  selected_pitch_device.display_name=d_name
  selected_pitch_device.parameters[1].show_in_mixer = true

end

---------------------------------------------------------------------------------------------


function get_pitch_device_properties(pitch_device)
  --Read the device title and set global variables for usage.
  --Routine is called when things change (instrument selection, DSP selection, track changes etc.)
  if pitch_device == nil then
    return
  end
  
  local properties = pitch_device.display_name:split("[^,]+")
  
  if properties ~= nil then
    if string.lower(properties[1]) ~= "pitch device" or properties[2] == nil then
      return false
    end

    if #properties > 0 then
      selected_instrument = tonumber(properties[2])

      if tonumber(properties[2]) < 1 then selected_instrument = 1 end
      if selected_instrument > #renoise.song().instruments then selected_instrument = 1 end

      sample_transpose_mode = tonumber(properties[3])
      if sample_transpose_mode < 1 then sample_transpose_mode = 1 end
      if sample_transpose_mode > 2 then sample_transpose_mode = 2 end

      if sample_transpose_mode == 2 then
        instrument_links[selected_instrument] = true
        link_sample_transpose(selected_instrument)
        reset_sample_offsets(selected_instrument)
      else
        instrument_links[selected_instrument] = false
        link_sample_transpose(selected_instrument)
      end
      
      pitch_range = tonumber(properties[4])
      if pitch_range < 1 then pitch_range = 1 end 
      
      if pitch_range > 12 and sample_transpose_mode == 1 then 
        pitch_range = 12 
        pitch_range_max = 12
      end
      
      if pitch_range > 120 and sample_transpose_mode ~= 1 then 
        pitch_range = 120 
        pitch_range_max = 120
      end

      selected_midi_device = tonumber(properties[5])
      if selected_midi_device < 1 then selected_midi_device = 1 end
      if selected_midi_device > #renoise.Midi.available_input_devices() then selected_midi_device = 1 end
      
      selected_channel = tonumber(properties[6])
      if selected_channel > 16 then selected_channel = 16 end
      
      if pa_gui ~= nil then
        change_from_renoise = true
          pa_gui.views['selected_instrument'].value = selected_instrument
          pa_gui.views['sample_transpose_mode'].value = sample_transpose_mode
          pa_gui.views['pitch_range'].value = pitch_range
          pa_gui.views['selected_midi_device'].value = selected_midi_device
          pa_gui.views['selected_channel'].value = selected_channel
        change_from_renoise = false
      end
    end
    return true
  end

  return false

end

---------------------------------------------------------------------------------------------

function add_pitch_device()
  -- Add another pitch bend device, name it properly and add default properties
  -- to the title.
  local song = renoise.song()
  local track=song.selected_track_index
  local device_path = 'Audio/Effects/Native/*Instr. MIDI Control'
  song.tracks[track]:insert_device_at(device_path, #song.tracks[track].devices+1)
  if song.tracks[track].devices[#song.tracks[track].devices].name == '*Instr. MIDI Control' then
    if string.sub(song.tracks[track].devices[#song.tracks[track].devices].display_name,1,20) == '*Instr. MIDI Control' then
      song.tracks[track].devices[#song.tracks[track].devices].display_name = 'Pitch Device'
      selected_pitch_device = song.tracks[track].devices[#song.tracks[track].devices]
      set_pitch_device_properties(true)
    end
  end

end
  
---------------------------------------------------------------------------------------------
  
function transpose_pitch_value(value)
  --routine is called by the Midi controller when the pitch bend controller is touched
  local song = renoise.song()
  local track=song.selected_track_index
  local track_device_idx = song.selected_device_index
  local track_device = song.tracks[track].devices[track_device_idx]
  local linked_instrument = get_pitch_device_instrument(track_device)
  change_from_tool = true
    if  linked_instrument ~= nil then
      if linked_instrument == song.selected_instrument_index then
        track_device.parameters[1].value = value
      end
    end 
  change_from_tool = false
end  

---------------------------------------------------------------------------------------------

function set_pitchbend_slider (track,device, linked_instrument, transpose_mode, pi_range)
  local song = renoise.song()
  if song.tracks[track].devices[device] ~= nil then
    selected_pitch_device = song.tracks[track].devices[device]
  else
    return
  end

  if not selected_pitch_device.is_active or change_from_config then
    return
  end
  
  local pattern = song.selected_pattern_index
  local instrument = song.instruments[linked_instrument].sample_envelopes
  local slider_max_value = 127

  if change_from_controller and song.transport.edit_mode and song.transport.record_parameter_mode == 2 then
    --Record the automation from the pitch-bend controller...
    --Only when edit mode is on and when record to envelopes is turned on.
    local automation_mode = 1
    local automation = nil
    for x = 1, #song.patterns[pattern].tracks[track].automation do
      if string.find(string.lower(song.patterns[pattern].tracks[track].automation[1].dest_device.display_name), "pitch device") ~= nil then
        automation_mode = song.patterns[pattern].tracks[track].automation[1].playmode
        automation = song.patterns[pattern].tracks[track].automation[1]
      end
    end
  
    selected_pitch_device.parameters[1]:record_value(selected_pitch_device.parameters[1].value)
  
    if automation ~= nil then
      automation.playmode = automation_mode
    end
  end
  
  if transpose_mode == 1 or transpose_mode == nil then
    --We are going to affect the pitch envelope point
    local factor = 100
  
    --Here are the translations calculated, if you pick a different device with a different 
    --maximum slider value, then change the slider_max_value.
    local pitch_center = (pi_range*factor)/2
    local pitch_envelope_range = 1200
    local pitch_range_one_percent = (pi_range/slider_max_value) *factor
    local slider_value = (pitch_range_one_percent* selected_pitch_device.parameters[1].value)
    local envelope_point_val = 0.5+(1/pitch_envelope_range*slider_value) - (1/pitch_envelope_range*pitch_center) 

    if slider_value <= pitch_center then
      local offset = (1/pitch_envelope_range*pitch_center) - (1/pitch_envelope_range*slider_value)
      envelope_point_val = 0.5-(offset)
    end
  
    --Add the calculated pitch point at the first position and also set the sustain node to that position.
    instrument.pitch:add_point_at(1,envelope_point_val)
    instrument.pitch.sustain_position = 1
    instrument.pitch.sustain_enabled = true
    instrument.pitch.enabled = true
  else
    --We are going to affect the sample transpose slider and the pitch LFO1
    local samples = song.instruments[linked_instrument].samples
    local pitch_center = (slider_max_value+1)/2
    local pitch_range_one_percent = (pi_range/pitch_center)
    local slider_value = pitch_range_one_percent* (selected_pitch_device.parameters[1].value -64)
    if selected_pitch_device.parameters[1].value <= pitch_center then
      slider_value = (pitch_range_one_percent* selected_pitch_device.parameters[1].value)
      slider_value = pitch_range - slider_value
      slider_value = 0 - slider_value                   
    end
    
    local fine_pitch = slider_value - math.floor(slider_value)
    local amount = 0
    slider_value = math.round(slider_value)
    instrument.pitch.lfo1.mode = 4
    instrument.pitch.lfo1.phase = 180
    instrument.pitch.lfo1.frequency = 0
    if selected_pitch_device.parameters[1].value < 63.5 then
      instrument.pitch.lfo1.phase = 0
      if fine_pitch < 0.5 then
        amount = 127 - ((64/0.5 * fine_pitch) + 64)
      else
        amount = 127 - ((64/0.5 * fine_pitch) -64)
      end
    elseif selected_pitch_device.parameters[1].value > 63.5 then
      if fine_pitch < 0.5 then
        amount = (64/0.5 * fine_pitch) + 64
      else
        amount = (64/0.5 * fine_pitch) -64
      end
    end

    if amount > 127 then 
      amount = 127 
    end
    if slider_value == pitch_range or slider_value == 0-pitch_range then
      amount = 0
    end
    if amount < 0 then amount = 0 end
    
    instrument.pitch.lfo1.amount = amount
    
    --this was shit to create because it required tons of notifier programmings!
    for x = 1, #samples do
      if soffset[x] == nil then
        soffset[x] = samples[x].transpose
      end
    end
    
    change_from_tool = true
    
      for x = 1, #samples do
        if samples[x].transpose + slider_value <= 120 or samples[x].transpose + slider_value >= -120 then
          samples[x].transpose = soffset[x] + slider_value
        end
      end
    change_from_tool = false
  end
  
end

---------------------------------------------------------------------------------------------

string.split = function(str, pattern)
  pattern = pattern or "[^%s]+"

  if pattern:len() == 0 then pattern = "[^%s]+" end

  local parts = {__index = table.insert}
  setmetatable(parts, parts)
  str:gsub(pattern, parts)
  setmetatable(parts, nil)
  parts.__index = nil
  return parts
end

---------------------------------------------------------------------------------------------

function math.round(value)
  local floor_val = math.floor(value)
  local float = value - floor_val

  if float < 0.5 then 
    return math.floor(value)
  else
    return math.ceil(value)
  end
end

---------------------------------------------------------------------------------------------

function start_pitch_monitor() 
  if renoise.song() == nil then
    return
  end

  if tool_dialog ~= nil then

    if tool_dialog.visible then
      tool_dialog:close()
    end
    tool_dialog = nil
  end

  for _ = 1, #renoise.song().instruments do
    instrument_links[_] = false
  end

  set_pitch_device_notifiers()
  set_pitch_device_notifier()
  monitor_pitch()
end

  --cover new song, this global notifier has to be set after the function definition
  --else you get tool errors. the start_pitch_monitor routine initiates all functionality
  --and monitors everything as soon as called, so therefore it is called only when a new
  --song is constructed or loaded.
  if not renoise.tool().app_new_document_observable:has_notifier(start_pitch_monitor) then
    renoise.tool().app_new_document_observable:add_notifier(start_pitch_monitor)
  end



--------------------------------------------------------------------------------
-- Menu entries
--------------------------------------------------------------------------------

renoise.tool():add_menu_entry {
  name = "DSP Device:Configure pitch device properties",
  invoke = open_main_dialog,
  active = watch_device_monitor
}

renoise.tool():add_menu_entry {
  name = "Mixer:Configure pitch device properties",
  invoke = open_main_dialog,
  active = watch_device_monitor
}

renoise.tool():add_menu_entry {
  name = "DSP Device:Add Favorite DSP:Pitch Device",
  invoke = add_pitch_device
}

renoise.tool():add_menu_entry {
  name = "DSP Device:Add Tool Device:Pitch Device",
  invoke = add_pitch_device
}

renoise.tool():add_menu_entry {
  name = "Mixer:Add Favorite DSP:Pitch Device",
  invoke = add_pitch_device
}

renoise.tool():add_menu_entry {
  name = "Mixer:Add Tool Device:Pitch Device",
  invoke = add_pitch_device
}

renoise.tool():add_menu_entry {
  name = "Track DSPs Chain:Add Favorite DSP:Pitch Device",
  invoke = add_pitch_device
}

renoise.tool():add_menu_entry {
  name = "Track DSPs Chain:Add Tool Device:Pitch Device",
  invoke = add_pitch_device
}

--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------

--[[
renoise.tool():add_keybinding {
  name = "Global:Tools:" .. tool_name.."...",
  invoke = show_dialog
}
--]]


--------------------------------------------------------------------------------
--      End of teh road...
--------------------------------------------------------------------------------

